@@ -39,12 +39,12 @@ class Connection(metaclass=ConnectionMeta):
     """
 
     __slots__ = ('_protocol', '_transport', '_loop',
-                 '_top_xact', '_uid', '_aborted',
+                 '_top_xact', '_aborted',
                  '_pool_release_ctr', '_stmt_cache', '_stmts_to_close',
                  '_listeners', '_server_version', '_server_caps',
                  '_intro_query', '_reset_query', '_proxy',
                  '_stmt_exclusive_section', '_config', '_params', '_addr',
-                 '_log_listeners')
+                 '_log_listeners', '_cancellations')
 
     def __init__(self, protocol, transport, loop,
                  addr: (str, int) or str,
@@ -54,9 +54,8 @@ class Connection(metaclass=ConnectionMeta):
         self._transport = transport
         self._loop = loop
         self._top_xact = None
-        self._uid = 0
         self._aborted = False
-        # Incremented very time the connection is released back to a pool.
+        # Incremented every time the connection is released back to a pool.
         # Used to catch invalid references to connection-related resources
         # post-release (e.g. explicit prepared statements).
         self._pool_release_ctr = 0
@@ -75,6 +74,7 @@ class Connection(metaclass=ConnectionMeta):
 
         self._listeners = {}
         self._log_listeners = set()
+        self._cancellations = set()
 
         settings = self._protocol.get_settings()
         ver_string = settings.server_version
@@ -264,19 +264,21 @@ class Connection(metaclass=ConnectionMeta):
         self._check_open()
         return await self._executemany(command, args, timeout)
 
-    async def _get_statement(self, query, timeout, *, named: bool=False):
-        statement = self._stmt_cache.get(query)
-        if statement is not None:
-            return statement
+    async def _get_statement(self, query, timeout, *, named: bool=False,
+                             use_cache: bool=True):
+        if use_cache:
+            statement = self._stmt_cache.get(query)
+            if statement is not None:
+                return statement
 
-        # Only use the cache when:
-        #  * `statement_cache_size` is greater than 0;
-        #  * query size is less than `max_cacheable_statement_size`.
-        use_cache = self._stmt_cache.get_max_size() > 0
-        if (use_cache and
-                self._config.max_cacheable_statement_size and
-                len(query) > self._config.max_cacheable_statement_size):
-            use_cache = False
+            # Only use the cache when:
+            #  * `statement_cache_size` is greater than 0;
+            #  * query size is less than `max_cacheable_statement_size`.
+            use_cache = self._stmt_cache.get_max_size() > 0
+            if (use_cache and
+                    self._config.max_cacheable_statement_size and
+                    len(query) > self._config.max_cacheable_statement_size):
+                use_cache = False
 
         if use_cache or named:
             stmt_name = self._get_unique_id('stmt')
@@ -289,12 +291,20 @@ class Connection(metaclass=ConnectionMeta):
             types, intro_stmt = await self.__execute(
                 self._intro_query, (list(ready),), 0, timeout)
             self._protocol.get_settings().register_data_types(types)
-            if not intro_stmt.name and not statement.name:
-                # The introspection query has used an anonymous statement,
-                # which has blown away the anonymous statement we've prepared
-                # for the query, so we need to re-prepare it.
-                statement = await self._protocol.prepare(
-                    stmt_name, query, timeout)
+            # The introspection query has used an anonymous statement,
+            # which has blown away the anonymous statement we've prepared
+            # for the query, so we need to re-prepare it.
+            need_reprepare = not intro_stmt.name and not statement.name
+        else:
+            need_reprepare = False
+
+        # Now that types have been resolved, populate the codec pipeline
+        # for the statement.
+        statement._init_codecs()
+
+        if need_reprepare:
+            await self._protocol.prepare(
+                stmt_name, query, timeout, state=statement)
 
         if use_cache:
             self._stmt_cache.put(query, statement)
@@ -328,8 +338,12 @@ class Connection(metaclass=ConnectionMeta):
 
         :return: A :class:`~prepared_stmt.PreparedStatement` instance.
         """
+        return await self._prepare(query, timeout=timeout, use_cache=False)
+
+    async def _prepare(self, query, *, timeout=None, use_cache: bool=False):
         self._check_open()
-        stmt = await self._get_statement(query, timeout, named=True)
+        stmt = await self._get_statement(query, timeout, named=True,
+                                         use_cache=use_cache)
         return prepared_stmt.PreparedStatement(self, query, stmt)
 
     async def fetch(self, query, *args, timeout=None) -> list:
@@ -423,6 +437,7 @@ class Connection(metaclass=ConnectionMeta):
             ...         'mytable', columns=('foo', 'bar'),
             ...         output='file.csv', format='csv')
             ...     print(result)
+            ...
             >>> asyncio.get_event_loop().run_until_complete(run())
             'COPY 100'
 
@@ -491,6 +506,7 @@ class Connection(metaclass=ConnectionMeta):
             ...         'SELECT foo, bar FROM mytable WHERE foo > $1', 10,
             ...         output='file.csv', format='csv')
             ...     print(result)
+            ...
             >>> asyncio.get_event_loop().run_until_complete(run())
             'COPY 10'
 
@@ -556,7 +572,8 @@ class Connection(metaclass=ConnectionMeta):
             ...     con = await asyncpg.connect(user='postgres')
             ...     result = await con.copy_to_table(
             ...         'mytable', source='datafile.tbl')
-            ....    print(result)
+            ...     print(result)
+            ...
             >>> asyncio.get_event_loop().run_until_complete(run())
             'COPY 140000'
 
@@ -621,7 +638,8 @@ class Connection(metaclass=ConnectionMeta):
             ...         'mytable', records=[
             ...             (1, 'foo', 'bar'),
             ...             (2, 'ham', 'spam')])
-            ....    print(result)
+            ...     print(result)
+            ...
             >>> asyncio.get_event_loop().run_until_complete(run())
             'COPY 2'
 
@@ -641,7 +659,7 @@ class Connection(metaclass=ConnectionMeta):
         intro_query = 'SELECT {cols} FROM {tab} LIMIT 1'.format(
             tab=tabname, cols=col_list)
 
-        intro_ps = await self.prepare(intro_query)
+        intro_ps = await self._prepare(intro_query, use_cache=True)
 
         opts = '(FORMAT binary)'
 
@@ -775,7 +793,7 @@ class Connection(metaclass=ConnectionMeta):
 
     async def set_type_codec(self, typename, *,
                              schema='public', encoder, decoder,
-                             binary=None, format='text'):
+                             format='text'):
         """Set an encoder/decoder pair for the specified data type.
 
         :param typename:
@@ -861,6 +879,7 @@ class Connection(metaclass=ConnectionMeta):
             ...         "SELECT '2 years 3 mons 1 day'::interval")
             ...     print(result)
             ...     print(datetime.datetime(2002, 1, 1) + result)
+            ...
             >>> asyncio.get_event_loop().run_until_complete(run())
             relativedelta(years=+2, months=+3, days=+1)
             2004-04-02 00:00:00
@@ -959,15 +978,30 @@ class Connection(metaclass=ConnectionMeta):
         """
         return not self._protocol.is_connected() or self._aborted
 
-    async def close(self):
-        """Close the connection gracefully."""
+    async def close(self, *, timeout=None):
+        """Close the connection gracefully.
+
+        :param float timeout:
+            Optional timeout value in seconds.
+
+        .. versionchanged:: 0.14.0
+           Added the *timeout* parameter.
+        """
         if self.is_closed():
             return
         self._mark_stmts_as_closed()
         self._listeners.clear()
         self._log_listeners.clear()
         self._aborted = True
-        await self._protocol.close()
+        try:
+            await self._protocol.close(timeout)
+        except Exception:
+            # If we fail to close gracefully, abort the connection.
+            self._aborted = True
+            self._protocol.abort()
+            raise
+        finally:
+            self._clean_tasks()
 
     def terminate(self):
         """Terminate the connection without waiting for pending data."""
@@ -976,22 +1010,45 @@ class Connection(metaclass=ConnectionMeta):
         self._log_listeners.clear()
         self._aborted = True
         self._protocol.abort()
+        self._clean_tasks()
 
-    async def reset(self):
+    async def reset(self, *, timeout=None):
         self._check_open()
         self._listeners.clear()
         self._log_listeners.clear()
         reset_query = self._get_reset_query()
+
+        if self._protocol.is_in_transaction() or self._top_xact is not None:
+            if self._top_xact is None or not self._top_xact._managed:
+                # Managed transactions are guaranteed to __aexit__
+                # correctly.
+                self._loop.call_exception_handler({
+                    'message': 'Resetting connection with an '
+                               'active transaction {!r}'.format(self)
+                })
+
+            self._top_xact = None
+            reset_query = 'ROLLBACK;\n' + reset_query
+
         if reset_query:
-            await self.execute(reset_query)
+            await self.execute(reset_query, timeout=timeout)
+
+    def _clean_tasks(self):
+        # Wrap-up any remaining tasks associated with this connection.
+        if self._cancellations:
+            for fut in self._cancellations:
+                if not fut.done():
+                    fut.cancel()
+            self._cancellations.clear()
 
     def _check_open(self):
         if self.is_closed():
             raise exceptions.InterfaceError('connection is closed')
 
     def _get_unique_id(self, prefix):
-        self._uid += 1
-        return '__asyncpg_{}_{}__'.format(prefix, self._uid)
+        global _uid
+        _uid += 1
+        return '__asyncpg_{}_{:x}__'.format(prefix, _uid)
 
     def _mark_stmts_as_closed(self):
         for stmt in self._stmt_cache.iter_statements():
@@ -1027,36 +1084,47 @@ class Connection(metaclass=ConnectionMeta):
             # so we ignore the timeout.
             await self._protocol.close_statement(stmt, protocol.NO_TIMEOUT)
 
-    def _cancel_current_command(self, waiter):
-        async def cancel():
-            try:
-                # Open new connection to the server
-                r, w = await connect_utils._open_connection(
-                    loop=self._loop, addr=self._addr, params=self._params)
-            except Exception as ex:
-                waiter.set_exception(ex)
-                return
+    async def _cancel(self, waiter):
+        r = w = None
 
-            try:
-                # Pack CancelRequest message
-                msg = struct.pack('!llll', 16, 80877102,
-                                  self._protocol.backend_pid,
-                                  self._protocol.backend_secret)
+        try:
+            # Open new connection to the server
+            r, w = await connect_utils._open_connection(
+                loop=self._loop, addr=self._addr, params=self._params)
 
-                w.write(msg)
-                await r.read()  # Wait until EOF
-            except ConnectionResetError:
-                # On some systems Postgres will reset the connection
-                # after processing the cancellation command.
-                pass
-            except Exception as ex:
+            # Pack CancelRequest message
+            msg = struct.pack('!llll', 16, 80877102,
+                              self._protocol.backend_pid,
+                              self._protocol.backend_secret)
+
+            w.write(msg)
+            await r.read()  # Wait until EOF
+        except ConnectionResetError as ex:
+            # On some systems Postgres will reset the connection
+            # after processing the cancellation command.
+            if r is None and not waiter.done():
                 waiter.set_exception(ex)
-            finally:
-                if not waiter.done():  # Ensure set_exception wasn't called.
-                    waiter.set_result(None)
+        except asyncio.CancelledError:
+            # There are two scenarios in which the cancellation
+            # itself will be cancelled: 1) the connection is being closed,
+            # 2) the event loop is being shut down.
+            # In either case we do not care about the propagation of
+            # the CancelledError, and don't want the loop to warn about
+            # an unretrieved exception.
+            pass
+        except Exception as ex:
+            if not waiter.done():
+                waiter.set_exception(ex)
+        finally:
+            self._cancellations.discard(
+                asyncio.Task.current_task(self._loop))
+            if not waiter.done():
+                waiter.set_result(None)
+            if w is not None:
                 w.close()
 
-        self._loop.create_task(cancel())
+    def _cancel_current_command(self, waiter):
+        self._cancellations.add(self._loop.create_task(self._cancel(waiter)))
 
     def _process_log_message(self, fields, last_query):
         if not self._log_listeners:
@@ -1116,13 +1184,6 @@ class Connection(metaclass=ConnectionMeta):
         caps = self._server_caps
 
         _reset_query = []
-        if self._protocol.is_in_transaction() or self._top_xact is not None:
-            self._loop.call_exception_handler({
-                'message': 'Resetting connection with an '
-                           'active transaction {!r}'.format(self)
-            })
-            self._top_xact = None
-            _reset_query.append('ROLLBACK;')
         if caps.advisory_locks:
             _reset_query.append('SELECT pg_advisory_unlock_all();')
         if caps.sql_close_all:
@@ -1189,6 +1250,53 @@ class Connection(metaclass=ConnectionMeta):
         else:
             self._drop_local_statement_cache()
 
+    async def reload_schema_state(self):
+        """Indicate that the database schema information must be reloaded.
+
+        For performance reasons, asyncpg caches certain aspects of the
+        database schema, such as the layout of composite types.  Consequently,
+        when the database schema changes, and asyncpg is not able to
+        gracefully recover from an error caused by outdated schema
+        assumptions, an :exc:`~asyncpg.exceptions.OutdatedSchemaCacheError`
+        is raised.  To prevent the exception, this method may be used to inform
+        asyncpg that the database schema has changed.
+
+        Example:
+
+        .. code-block:: pycon
+
+            >>> import asyncpg
+            >>> import asyncio
+            >>> async def change_type(con):
+            ...     result = await con.fetch('SELECT id, info FROM tbl')
+            ...     # Change composite's attribute type "int"=>"text"
+            ...     await con.execute('ALTER TYPE custom DROP ATTRIBUTE y')
+            ...     await con.execute('ALTER TYPE custom ADD ATTRIBUTE y text')
+            ...     await con.reload_schema_state()
+            ...     for id_, info in result:
+            ...         new = (info['x'], str(info['y']))
+            ...         await con.execute(
+            ...             'UPDATE tbl SET info=$2 WHERE id=$1', id_, new)
+            ...
+            >>> async def run():
+            ...     # Initial schema:
+            ...     # CREATE TYPE custom AS (x int, y int);
+            ...     # CREATE TABLE tbl(id int, info custom);
+            ...     con = await asyncpg.connect(user='postgres')
+            ...     async with con.transaction():
+            ...         # Prevent concurrent changes in the table
+            ...         await con.execute('LOCK TABLE tbl')
+            ...         await change_type(con)
+            ...
+            >>> asyncio.get_event_loop().run_until_complete(run())
+
+        .. versionadded:: 0.14.0
+        """
+        # It is enough to clear the type cache only once, not in each
+        # connection in the pool.
+        self._protocol.get_settings().clear_type_cache()
+        self._drop_global_statement_cache()
+
     async def _execute(self, query, args, limit, timeout, return_status=False):
         with self._stmt_exclusive_section:
             result, _ = await self.__execute(
@@ -1230,6 +1338,15 @@ class Connection(metaclass=ConnectionMeta):
                     after = time.monotonic()
                     timeout -= after - before
 
+        except exceptions.OutdatedSchemaCacheError:
+            # This exception is raised when we detect a difference between
+            # cached type's info and incoming tuple from the DB (when a type is
+            # changed by the ALTER TYPE).
+            # It is not possible to recover (the statement is already done at
+            # the server's side), the only way is to drop our caches and
+            # reraise the exception to the caller.
+            await self.reload_schema_state()
+            raise
         except exceptions.InvalidCachedStatementError:
             # PostgreSQL will raise an exception when it detects
             # that the result type of the query has changed from
@@ -1357,6 +1474,7 @@ async def connect(dsn=None, *,
         ...     con = await asyncpg.connect(user='postgres')
         ...     types = await con.fetch('SELECT * FROM pg_type')
         ...     print(types)
+        ...
         >>> asyncio.get_event_loop().run_until_complete(run())
         [<Record typname='bool' typnamespace=11 ...
 
@@ -1468,7 +1586,8 @@ class _StatementCache:
 
         if entry._statement.closed:
             # Happens in unittests when we call `stmt._state.mark_closed()`
-            # manually.
+            # manually or when a prepared statement closes itself on type
+            # cache error.
             self._entries.pop(query)
             self._clear_entry_callback(entry)
             return
@@ -1583,6 +1702,13 @@ def _detect_server_capabilities(server_version, connection_settings):
         plpgsql = False
         sql_reset = False
         sql_close_all = False
+    elif hasattr(connection_settings, 'crate_version'):
+        # CrateDB detected.
+        advisory_locks = False
+        notifications = False
+        plpgsql = False
+        sql_reset = False
+        sql_close_all = False
     else:
         # Standard PostgreSQL server assumed.
         advisory_locks = True
@@ -1598,3 +1724,6 @@ def _detect_server_capabilities(server_version, connection_settings):
         sql_reset=sql_reset,
         sql_close_all=sql_close_all
     )
+
+
+_uid = 0
