@@ -817,6 +817,16 @@ class TestCodecs(tb.ConnectedTestCase):
                 ValueError, 'expected 0, 1 or 2 elements'):
             await self.con.fetch("SELECT $1::int4range", (0, 2, 3))
 
+        cases = [(asyncpg.Range(0, 1), asyncpg.Range(0, 1), 1),
+                 (asyncpg.Range(0, 1), asyncpg.Range(0, 2), 2),
+                 (asyncpg.Range(empty=True), asyncpg.Range(0, 2), 2),
+                 (asyncpg.Range(empty=True), asyncpg.Range(empty=True), 1),
+                 (asyncpg.Range(0, 1, upper_inc=True), asyncpg.Range(0, 1), 2),
+                 ]
+        for obj_a, obj_b, count in cases:
+            dic = {obj_a: 1, obj_b: 2}
+            self.assertEqual(len(dic), count)
+
     async def test_extra_codec_alias(self):
         """Test encoding/decoding of a builtin non-pg_catalog codec."""
         await self.con.execute('''
@@ -1011,7 +1021,7 @@ class TestCodecs(tb.ConnectedTestCase):
         """Test overriding core codecs."""
         import json
 
-        conn = await self.cluster.connect(database='postgres', loop=self.loop)
+        conn = await self.connect()
         try:
             def _encoder(value):
                 return json.dumps(value).encode('utf-8')
@@ -1035,7 +1045,7 @@ class TestCodecs(tb.ConnectedTestCase):
         """Test overriding core codecs."""
         import json
 
-        conn = await self.cluster.connect(database='postgres', loop=self.loop)
+        conn = await self.connect()
         try:
             def _encoder(value):
                 return json.dumps(value)
@@ -1087,7 +1097,7 @@ class TestCodecs(tb.ConnectedTestCase):
             ('interval', (2, 3, 1), '2 mons 3 days 00:00:00.000001')
         ]
 
-        conn = await self.cluster.connect(database='postgres', loop=self.loop)
+        conn = await self.connect()
 
         def _encoder(value):
             return tuple(value)
@@ -1129,6 +1139,27 @@ class TestCodecs(tb.ConnectedTestCase):
         finally:
             await conn.close()
 
+    async def test_timetz_encoding(self):
+        try:
+            async with self.con.transaction():
+                await self.con.execute("SET TIME ZONE 'America/Toronto'")
+                # Check decoding:
+                row = await self.con.fetchrow(
+                    'SELECT extract(epoch from now()) AS epoch, '
+                    'now()::date as date, now()::timetz as time')
+                result = datetime.datetime.combine(row['date'], row['time'])
+                expected = datetime.datetime.fromtimestamp(row['epoch'],
+                                                           tz=result.tzinfo)
+                self.assertEqual(result, expected)
+
+                # Check encoding:
+                res = await self.con.fetchval(
+                    'SELECT now() = ($1::date + $2::timetz)',
+                    row['date'], row['time'])
+                self.assertTrue(res)
+        finally:
+            await self.con.execute('RESET ALL')
+
     async def test_composites_in_arrays(self):
         await self.con.execute('''
             CREATE TYPE t AS (a text, b int);
