@@ -11,6 +11,8 @@ import inspect
 import os
 import platform
 import random
+import sys
+import textwrap
 import time
 import unittest
 
@@ -31,15 +33,16 @@ else:
 
 class SlowResetConnection(pg_connection.Connection):
     """Connection class to simulate races with Connection.reset()."""
-    async def reset(self):
+    async def reset(self, *, timeout=None):
         await asyncio.sleep(0.2, loop=self._loop)
-        return await super().reset()
+        return await super().reset(timeout=timeout)
 
 
-class SlowResetConnectionPool(pg_pool.Pool):
-    async def _connect(self, *args, **kwargs):
-        return await pg_connection.connect(
-            *args, connection_class=SlowResetConnection, **kwargs)
+class SlowCancelConnection(pg_connection.Connection):
+    """Connection class to simulate races with Connection._cancel()."""
+    async def _cancel(self, waiter):
+        await asyncio.sleep(0.2, loop=self._loop)
+        return await super()._cancel(waiter)
 
 
 class TestPool(tb.ConnectedTestCase):
@@ -194,6 +197,7 @@ class TestPool(tb.ConnectedTestCase):
             self.assertIn(repr(con._con), repr(con))  # Test __repr__.
 
             ps = await con.prepare('SELECT 1')
+            txn = con.transaction()
             async with con.transaction():
                 cur = await con.cursor('SELECT 1')
                 ps_cur = await ps.cursor()
@@ -232,6 +236,14 @@ class TestPool(tb.ConnectedTestCase):
 
                 c.forward(1)
 
+        for meth in ('start', 'commit', 'rollback'):
+            with self.assertRaisesRegex(
+                    asyncpg.InterfaceError,
+                    r'cannot call Transaction\.{meth}.*released '
+                    r'back to the pool'.format(meth=meth)):
+
+                getattr(txn, meth)()
+
         await pool.close()
 
     async def test_pool_12(self):
@@ -351,12 +363,12 @@ class TestPool(tb.ConnectedTestCase):
             self.cluster.trust_local_connections()
             self.cluster.reload()
 
-    async def test_pool_handles_cancel_in_release(self):
+    async def test_pool_handles_task_cancel_in_release(self):
         # Use SlowResetConnectionPool to simulate
         # the Task.cancel() and __aexit__ race.
         pool = await self.create_pool(database='postgres',
                                       min_size=1, max_size=1,
-                                      pool_class=SlowResetConnectionPool)
+                                      connection_class=SlowResetConnection)
 
         async def worker():
             async with pool.acquire():
@@ -372,6 +384,27 @@ class TestPool(tb.ConnectedTestCase):
         # Check that the connection has been returned to the pool.
         self.assertEqual(pool._queue.qsize(), 1)
 
+    async def test_pool_handles_query_cancel_in_release(self):
+        # Use SlowResetConnectionPool to simulate
+        # the Task.cancel() and __aexit__ race.
+        pool = await self.create_pool(database='postgres',
+                                      min_size=1, max_size=1,
+                                      connection_class=SlowCancelConnection)
+
+        async def worker():
+            async with pool.acquire() as con:
+                await con.execute('SELECT pg_sleep(10)')
+
+        task = self.loop.create_task(worker())
+        # Let the worker() run.
+        await asyncio.sleep(0.1, loop=self.loop)
+        # Cancel the worker.
+        task.cancel()
+        # Wait to make sure the cleanup has completed.
+        await asyncio.sleep(0.5, loop=self.loop)
+        # Check that the connection has been returned to the pool.
+        self.assertEqual(pool._queue.qsize(), 1)
+
     async def test_pool_no_acquire_deadlock(self):
         async with self.create_pool(database='postgres',
                                     min_size=1, max_size=1,
@@ -492,9 +525,10 @@ class TestPool(tb.ConnectedTestCase):
         methods = [test_fetch, test_fetchrow, test_fetchval,
                    test_execute, test_execute_with_arg]
 
-        for method in methods:
-            with self.subTest(method=method.__name__):
-                await run(200, method)
+        with tb.silence_asyncio_long_exec_warning():
+            for method in methods:
+                with self.subTest(method=method.__name__):
+                    await run(200, method)
 
     async def test_pool_connection_execute_many(self):
         async def worker(pool):
@@ -638,6 +672,75 @@ class TestPool(tb.ConnectedTestCase):
         await con.close()
         await pool.close()
 
+    @unittest.skipIf(sys.version_info[:2] < (3, 6), 'no asyncgen support')
+    async def test_pool_handles_transaction_exit_in_asyncgen_1(self):
+        pool = await self.create_pool(database='postgres',
+                                      min_size=1, max_size=1)
+
+        locals_ = {}
+        exec(textwrap.dedent('''\
+            async def iterate(con):
+                async with con.transaction():
+                    for record in await con.fetch("SELECT 1"):
+                        yield record
+        '''), globals(), locals_)
+        iterate = locals_['iterate']
+
+        class MyException(Exception):
+            pass
+
+        with self.assertRaises(MyException):
+            async with pool.acquire() as con:
+                async for _ in iterate(con):  # noqa
+                    raise MyException()
+
+    @unittest.skipIf(sys.version_info[:2] < (3, 6), 'no asyncgen support')
+    async def test_pool_handles_transaction_exit_in_asyncgen_2(self):
+        pool = await self.create_pool(database='postgres',
+                                      min_size=1, max_size=1)
+
+        locals_ = {}
+        exec(textwrap.dedent('''\
+            async def iterate(con):
+                async with con.transaction():
+                    for record in await con.fetch("SELECT 1"):
+                        yield record
+        '''), globals(), locals_)
+        iterate = locals_['iterate']
+
+        class MyException(Exception):
+            pass
+
+        with self.assertRaises(MyException):
+            async with pool.acquire() as con:
+                iterator = iterate(con)
+                async for _ in iterator:  # noqa
+                    raise MyException()
+
+            del iterator
+
+    @unittest.skipIf(sys.version_info[:2] < (3, 6), 'no asyncgen support')
+    async def test_pool_handles_asyncgen_finalization(self):
+        pool = await self.create_pool(database='postgres',
+                                      min_size=1, max_size=1)
+
+        locals_ = {}
+        exec(textwrap.dedent('''\
+            async def iterate(con):
+                for record in await con.fetch("SELECT 1"):
+                    yield record
+        '''), globals(), locals_)
+        iterate = locals_['iterate']
+
+        class MyException(Exception):
+            pass
+
+        with self.assertRaises(MyException):
+            async with pool.acquire() as con:
+                async with con.transaction():
+                    async for _ in iterate(con):  # noqa
+                        raise MyException()
+
 
 @unittest.skipIf(os.environ.get('PGHOST'), 'using remote cluster for testing')
 class TestHotStandby(tb.ConnectedTestCase):
@@ -656,7 +759,8 @@ class TestHotStandby(tb.ConnectedTestCase):
 
         try:
             con = cls.loop.run_until_complete(
-                cls.master_cluster.connect(database='postgres', loop=cls.loop))
+                cls.master_cluster.connect(
+                    database='postgres', user='postgres', loop=cls.loop))
 
             cls.loop.run_until_complete(
                 con.execute('''
@@ -696,8 +800,9 @@ class TestHotStandby(tb.ConnectedTestCase):
     async def test_standby_pool_01(self):
         for n in {1, 3, 5, 10, 20, 100}:
             with self.subTest(tasksnum=n):
-                pool = await self.create_pool(database='postgres',
-                                              min_size=5, max_size=10)
+                pool = await self.create_pool(
+                    database='postgres', user='postgres',
+                    min_size=5, max_size=10)
 
                 async def worker():
                     con = await pool.acquire()
@@ -710,7 +815,7 @@ class TestHotStandby(tb.ConnectedTestCase):
 
     async def test_standby_cursors(self):
         con = await self.standby_cluster.connect(
-            database='postgres', loop=self.loop)
+            database='postgres', user='postgres', loop=self.loop)
 
         try:
             async with con.transaction():
